#-*-coding:Utf-8 -*-
# script to read Evaluator files and make plots

from __future__ import print_function

import evaltools as evt
import os
from matplotlib.colors import ListedColormap
from datetime import date

#################
# define params #
#################

startDate = date(2018, 11, 19)
endDate = date(2018, 11, 23)

liste_pol = ['PM10']
models_list = ['mod1', 'mod2']

threshold = {'PM10': 20,
             'O3': 180,
             'PM25': 25,
             'NO2': 200}

correc_units = {'PM25': 1.,
                'PM10': 1.,
                'O3': 1.,
                'NO2': 1.}

rootPath = './'
evt_path = rootPath+"dump_files/"
output = rootPath+"outputs/"

#################
# read listings #
#################

period = "\n({startdate} to {enddate})".format(
                       startdate=startDate.strftime("%Y-%m-%d"),
                       enddate=endDate.strftime("%Y-%m-%d"))

for polluant in liste_pol:
    # threshold value
    thr = threshold.get(polluant, 100)

    # make output directory if needed
    outpath = output + polluant + "/"
    if not os.path.exists(outpath):
        os.makedirs(outpath)

    print("\nLoading Evaluator objects for "+polluant+" ...")
    evt_list = []
    evt_urban = []

    for model in models_list:
        # load Evaluator objects from previous save
        evt1 = evt.evaluator.load(evt_path+polluant+"_"+model+"_2018.evt")

        # reduce studied period, if needed
        # evt1 = evt1.subPeriod(startDate,endDate)
        print(evt1.summary())

        # save raw scores for each forecast day
        evt1.stationScores(['NbObs', 'MeanBias', 'RMSE', 'PearsonR'],
                           outputFile=(outpath + "/stationScores_" +
                                       evt1.model + "_{forecastDay}.txt"),
                           availability_ratio=0.25)

        # compute daily mean/max values
        if polluant == 'O3' or polluant == 'NO2':
            dailyType = 'max'
            obj1 = evt1.dailyMax(availability_ratio=.25)
        else:
            dailyType = 'mean'
            obj1 = evt1.dailyMean(availability_ratio=.25)
        evt_list.append(obj1)

        # get urban stations in a separate list
        subevt = evt1.stationSubList(
                      evt1.stations[
                           evt1.stations['area'] == 'urban'
                           ].index.tolist(), False)
        evt_urban.append(subevt)

    ##### objects are loaded : it is time to plot #####

    print("~~~ Plotting ~~~")
    print("Line exceedances")
    evt.plotting.plot_lineExceedances(evt_list, thr,
                                      outputFile=outpath + "/lineExceedances",
                                      title=(polluant
                                             + " threshold exceedances for "
                                             + str(thr) + " ug/m3, daily "
                                             + dailyType),
                                      date_format='%d %m %Y')

    print("Time series")
    sta = evt_list[0].stations.index.values[0]
    evt.plotting.plot_timeSeries(evt_list, station_list=[sta],
                                 outputFile=outpath + "/timeSeries_" + sta,
                                 start_end=[startDate, endDate],
                                 title=("Time series of " + dailyType
                                        + " observed and modelled "
                                        + polluant + " data\nStation " + sta),
                                 file_formats=['png'], black_axes=True)

    print("Bar scores")
    sc = "PearsonR"
    evt.plotting.plot_barScores(evt_list, [sc],
                                outputFile=outpath + '/barScores_' + sc,
                                title=(polluant + " daily " + dailyType
                                       + " " + sc
                                       + period),
                                availability_ratio=.25)

    print("Bar scores per region")
    sc = "RMSE"
    subregions = ['all', ['FR0', 'FR1'], ['FR2', 'FR3']]
    xticks = ['All\n({nbStations} stations)',
              "FR0&1\n({nbStations} stations)",
              "FR2&3\n({nbStations} stations)"]
    evt.plotting.plot_barScores(evt_list, [sc],
                                outputFile=outpath + '/barScoresReg_' + sc,
                                subregions=subregions,
                                xticksLabels=xticks,
                                title=(polluant + " daily " + dailyType
                                       + " " + sc
                                       + period),
                                availability_ratio=.25)

    print("Bar scores for urban stations")
    sc = "MeanBias"
    evt.plotting.plot_barScores(evt_urban, [sc],
                                outputFile=outpath + '/barScores_Bias_urban',
                                title=(polluant + " daily " + dailyType
                                       + " " + sc + " for urban sites"
                                       + period),
                                availability_ratio=.25)

    print("Taylor diagram")
    evt.plotting.plot_taylorDiagram(evt_list, norm=False,
                                    outputFile=outpath + "/taylorDiagram",
                                    title=(polluant + " Taylor diagram, daily "
                                           + dailyType + period),
                                    file_formats=['png'], threshold=.25)

    print("Contingency table")
    evt.plotting.plot_barContingencyTable(evt_list, thr,
                                          outputFile=outpath+"/perfscore",
                                          title=(polluant
                                                 + " contingency table for "
                                                 + str(thr) + " ug/m3, daily "
                                                 + dailyType + period))

    print("Station scores")
    sc = "PearsonR"
    for obj in evt_list:
        evt.plotting.plot_stationScores(
            obj, sc,
            outputFile=outpath + "/mapstation_Corr_" + obj.model,
            title=("Correlation for " + polluant + ", daily "
                   + dailyType + ", " + obj.model + period),
            file_formats=['png'],
            availability_ratio=0.25,
            bbox=[-5.5, 13., 40., 55.],
            vmin=0, vmax=1, extend='min',
            cmap='RdYlGn', boundary_resolution='50m',
            land_color='beige', sea_color='lightsteelblue',
            point_size=15,
            mark_by=['area', {'urban': 's',
                              'rural': 'o',
                              'suburban': '^'}]
            )
